

#ifndef AVRREGISTER
	#define AVRREGISTER
/*  14. 10. 2011  Christof Ermer - Regensburg
Beschreibt Funktion und Aufgabe der einzelnen Bits in Registern
so kann im Programm mit lesbaren Namen anstelle von Zahlen gearbeitet Werden
Einzelne Makros (), die daraus abgeliette werden,erleichtern die Programmierung
Registernamen stehen zuerst Z.B: "TCCR0_"
*/

// ******************************
#define BIT_0	0x01
#define BIT_1	0x02
#define BIT_2	0x04
#define BIT_3	0x08
#define BIT_4	0x10
#define BIT_5	0x20
#define BIT_6	0x40
#define BIT_7	0x80


//SFIOR Special FunctionIO Reg
//Modes fr ADC Trigger
#define SFIOR_ADC_ANALOG_TRIGGER 0x20


// ******************************
// SPEZIFISCHE DEFINITIONEN; JE NACH HARDWARE
//BEI 16 MHz Quarz = F_CPU / Teiler
// QUARZ/DIVIDE = FREQUENZ
#define TIMER_DIVIDE_FRQ_1	((float)F_CPU)	// 16000000UL
#define TIMER_DIVIDE_FRQ_8	(F_CPU / 8.0) //2000000UL		
#define TIMER_DIVIDE_FRQ_64	(F_CPU / 64.0) //250000UL
#define TIMER_DIVIDE_FRQ_256	(F_CPU / 256.0) //62500U
#define TIMER_DIVIDE_FRQ_1024	(F_CPU / 1024.0) //15625U

#define T1_FRQ_MAXIMAL_FRQ	16000000
#define T1_FRQ_MINIMAL_FRQ	0.238422217  // 16^6/1024/65535


// ******************************
//TIMER REGISTER
#define TCCR0_FAST_PWM	0x48	//Mode 3
#define TCCR0_OC0_MODE	0x20	//Clear Timer on Compare Match
#define TCCR0_OC0_DISCONNECT_MODE	0x00
#define TCCR0_CTC_MODE	0x08
#define TCCR0_CLK_STOP	0x00	
#define TCCR0_CLK_STOP_MASK	0xF8	
#define TCCR0_CLK_1		0x01	//62500 PWM
#define TCCR0_CLK_8		0x02	//7812 Hz PWM
#define TCCR0_CLK_64	0x03	//976,56Hz PWM
#define TCCR0_CLK_256	0x04	//244,14Hz PWM
#define TCCR0_CLK_1024	0x05	//~61,035 PWM
#define TCCR0_EXT_CLK_FALL_T0	0x06	
#define TCCR0_EXT_CLK_RISE_T0	0x07

// ******************************
//TCCR1A
#define TCCR1A_CTC_MODE			0x00	//WGM11 WGM10  = 0
#define TCCR1A_FAST_PWM_ICR1	0x02 //WGM11
#define TCCR1A_FAST_PWM_8BIT	0x01 //WGM11 WGM10 n TCCR1A
#define TCCR1A_FAST_PWM_9BIT	0x02 //WGM11 WGM10 n TCCR1A
#define TCCR1A_FAST_PWM_10BIT	0x03 //WGM11 WGM10 n TCCR1A
#define TCCR1A_CLEAR_WGMX_MASK 0xFC // Lsche WGM 11/10
#define TCCR1A_PWM_CLR_OC1A	0x80	//Clear OC1A=C1B 

// ******************************
//#define TCCR1B_CLOCKSEL_CTC1	0x08	//Clear Timer/Counter  on Compare Match Timer/Counter 1
#define TCCR1B_CLOCKSEL_STOP	0x00	//Clock Select Bits
#define TCCR1B_CLOCKSEL_MSK		0xF8	//zum Stoppen der CLK
#define TCCR1B_CTC_MODE			0x08	//WGM12 ClearTimeronCompare
#define TCCR1B_FAST_PWM_XBIT	0x08	//WGM12 only
#define TCCR1B_CLOCKSEL_CPU_1	0x01	//Clock Select Bits
#define TCCR1B_CLOCKSEL_CPU_8	0x02	//Clock Select Bits
#define TCCR1B_CLOCKSEL_CPU_64	0x03	//Clock Select Bits
#define TCCR1B_CLOCKSEL_CPU_256	0x04	//Clock Select Bits
#define TCCR1B_CLOCKSEL_CPU_1024	0x05	//Clock Select Bits
#define TCCR1B_CLK_STOP_MSK		0xF8//TCCR1A

// ******************************
//TCCR2
#define TCCR2_WGM_NORMAL	0x00
#define TCCR2_CTC_MODE		0x08 //WGM12 0000 1000
#define TCCR2_CTC_FAST_PWM	0x08 //WGM12 0000 1000
#define TCCR2_COM_NORMALPORT	0x00
#define TCCR2_CLK_STOP	0x00
#define TCCR2_FAST_PWM	0x48	//Mode 3
#define TCCR2_CLK_1		0x01
#define TCCR2_CLK_8		0x02
#define TCCR2_CLK_32		0x03
#define TCCR2_CLK_64		0x04
#define TCCR2_CLK_128	0x05
#define TCCR2_CLK_256	0x06
#define TCCR2_CLK_1024	0x07
#define TCCR2_CLK_STOP	0x00
#define TCCR2_CLK_STOP__MSK	0xF8	
//TIMER2 WIRD BERECHNET /z.B:OverFLow SYSCLK /  256 / TCCR2_CLK_64 =  OVL/Sec

//TIMER
#define TIMSK_TOIE0_FLAG	0x01
#define TIMSK_OCIE0_FLAG	0x02
#define TIMSK_TOIE1_FLAG	0x04
#define TIMSK_OCIE1B_FLAG	0x08
#define TIMSK_OCIE1A_FLAG	0x10
#define TIMSK_TICIE1_FLAG	0x20
#define TIMSK_TOIE2_FLAG	0x40 // ( 1<< TOIE2 )
#define TIMSK_OCIE2_FLAG	0x80

#define TIMSK_T0_CLR_ALL_MASK 0b11111100
#define TIMSK_T1_CLR_ALL_MASK 0b11000011
#define TIMSK_T2_CLR_ALL_MASK 0b00111111




// ******************************
//Global Interrrupt CTRL REG GICR
#define GICR_INT1_FLAG	0x80	//INT1
#define GICR_INT0_FLAG	0x40	//INT0	
#define GICR_INT2_FLAG	0x20	//INT2

#define GICR_INT0_ON()	(GICR |= _BV(INT0)) //_BV(INT0)
#define GICR_INT0_OFF()	(GICR &= ~(_BV(INT0)))
#define GICR_INT1_ON()	(GICR |= _BV(INT1))
#define GICR_INT1_OFF()	(GICR &= ~(_BV(INT1)))
#define GICR_INT_0u1_OFF()	(GICR &= 0x3F)  //SIEHE DATENBLATT


// ******************************
//GIFR  I_FLAG //Siehe Datenblatt = letzten beiden Bits
#define GIFR_INTF1_FLAG	0x80	//INT1
#define GIFR_INTF0_FLAG	0x40	//INT0
#define GIFR_INTF2_FLAG	0x20	//INT2

#define GIFR_INTF0_SET()	(GIFR |= GIFR_INTF0_FLAG)	//_BV(INTF0)
#define GIFR_INTF0_CLR()	(GIFR &= ~GIFR_INTF0_FLAG)	//~_BV(INTF0)
#define GIFR_INTF1_CLR()	(GIFR |= GIFR_INTF1_FLAG) //_BV(INTF1)
#define GIFR_INTF1_SET()	(GIFR &= ~GIFR_INTF1_FLAG)	//~_BV(INTF1)
#define GIFR_INTF2_CLR()	(GIFR |= GIFR_INTF2_FLAG) //_BV(INTF1)
#define GIFR_INTF2_SET()	(GIFR &= ~GIFR_INTF2_FLAG)	//~_BV(INTF1)
#define GIFR_INTF_0u1u2_CLR()	(GIFR &= 0x1F) //Siehe Datenblatt=Bits:0xE0

// ******************************
#define TIFR_TOV0_TIMER_OVERFLOW_FLAG	0x01
#define TIFR_OCF0_OUT_PCOMPARE_FLAG		0x02
#define TIFR_TOV1_TIMER_OVERFLOW_FLAG	0x04
#define TIFR_OCF1A_OUTP_COMPARE_A_FLAG	0x10
#define TIFR_OCF1B_OUTP_COMPARE_B_FLAG	0x08
#define TIFR_TOV2_TIMER_OVERFLOW_FLAG	0x40
#define TIFR_OCF2_OUTP_COMPARE_FLAG		0x80

// ******************************
//INTERUPT CTRL REG //NICHT mit MCUCSR verwechseln !
//INTERRUPT MODE
//#define MCUCR_INT0_LOWLEVEL_CLR_MSK	0xFC

#define MCUCR_ADC_NOISE_RUDCTION	0x10	
#define MCUCR_LOWLEVEL_INT0		0x00	
#define MCUCR_ANYCHANGE_INT0		0x01 //=_BV(ISC01) 	
#define MCUCR_FALLING_EDGE_INT0	0x02	
#define MCUCR_RISING_EDGE_INT0	0x03

//#define MCUCR_INT1_LOWLEVEL_CLR_MSK	0xF3
#define MCUCR_LOWLEVEL_INT1		0x00	
#define MCUCR_ANYCHANGE_INT1		0x04 //=_BV(ISC10) 	
#define MCUCR_FALLING_EDGE_INT1	0x08	
#define MCUCR_RISING_EDGE_INT1	0x0C
//------------
#define MCUCR_CLEAR_MASK_INT0	0xFC
#define MCUCR_CLEAR_MASK_INT1	0xF3

#define GATE_ANYCHANGE_INT1_MODE()	(MCUCR |= MCUCR_ANYCHANGE_INT1)


// ******************************
//ADC //AT8 Prfen..
#define ADMUX_MUX0	0
#define ADMUX_MUX1	1
#define ADMUX_MUX2	2
#define ADMUX_MUX3	3
#define ADMUX_MUX4	4
#define ADMUX_MUX5	5
#define ADMUX_MUX_1_23V	0xE0
#define ADMUX_MUX_GND	0xF0
#define ADMUX_ADLAR	0x20
#define ADMUX_REFS0	0x40
#define ADMUX_REFS1	0x80
#define ADC_MUX__MSK	0x0F

//PRESCALER
#define ADCSRA_ADPS0	0x01
#define ADCSRA_ADPS1	0x02
#define ADCSRA_ADPS2	0x04
#define ADCSRA_ADIE		0x08
#define ADCSRA_ADIF		0x10		//INTERRRUPT FLAG
#define ADCSRA_ADFR		0x20	//FREE RUNNNG SEL
#define ADCSRA_ADSC		0x40	//START  //while (ADCSRA & _BV(ADSC) ) 
#define ADCSRA_ADEN		0x80	//Enable

#define  ADCSRA_PRESCALER_2		0x01
#define  ADCSRA_PRESCALER_4		0x02
#define  ADCSRA_PRESCALER_8		0x03
#define  ADCSRA_PRESCALER_16	0x04
#define  ADCSRA_PRESCALER_32	0x05
#define  ADCSRA_PRESCALER_64	0x06
#define  ADCSRA_PRESCALER_128	0x07

//ADC intern 10Bit 	
#define ADC_MUX_MASK			0x07	//Fr internen ADC
#define ADC10BIT_NTC_CHAN		7
#define ADC10BIT_INDEX_CHAN	6


// ******************************
//EEPROM
// ******************************
#define EEPROM_SECTION  __attribute__ ((section (".eeprom")))
#define IEEPROM_SIZE	512	//Interne EEPROM
#define EE_DUMMYLOAD 0x5AA5 //unwahrsch. wert 0101 1010 1010 0101
//If using 8 block by 512 max = 64 Byte/block
//#define IEEPROM_SECTIONSIZE	64   //by 8 Section

// ******************************
//HARDWARE nahes
// ******************************
//INTERRUPT
#define INTx_INPORT	PIND
#define INT1_MASK		0x08  //PD3
#define INT0_MASK		0x04  //PD2
#define INT0_PIN_NR	PD2
#define INT1_PIN_NR	PD3

#define INT1_PRESSED_DETECT	0x01
#define INT1_PRESSED				0x02
#define INT1_RELEASED_DETECT	0x04
#define INT1_RELEASED			0x08
//---------
#define INT1_INT_EVENT			0x40 // SOnderrolle um zu unterscheiden, ob Taste bei Prellen wieder Frei 
#define INT1_VALID				0x80
#define INT1_CLR_ALL_SW_FLAGS_MASK	0xC0 //AUSSER VALID.. AUSSER INT
#define INT1_CLR_INT_AND_ALL_SW_FLAGS_MASK	0x80 //AUSSER VALID..MIT INT  !!
//gsKey.ucKeyFlag   gsKey.ulEventTime   gsKey.ucKey

#define INT1_SWITCH_RELEASE_DELAY_TICKS	 244	//~1/5 Sek1INTERRUPTS_PER_SEC_T2 244
#define INT1_SWITCH_PRESSED_DELAY_TICKS  10	//Beuetet 5MS mus mindest der selbe Wert gelesen sein.


//WERT IN ADCW
#define I2C_CLOCK_BIT	PC0
#define I2C_DATA_BIT	PC1

//PORTB
#define TWI_DIRREG		DDRC
#define TWI_OUT_PORT		PORTC
#define TWI_IN_PORT		PINC
#define TWI_SDA			PC4		//PC4	OUT
#define TWI_SCL			PC5		//PC5	IN
/*
#define Clr_TWI_SDA()	 	TWI_OUT_PORT &= ~PC_SDA_BIT;	 
#define Set_TWI_SDA()	 	TWI_OUT_PORT |= PC_SDA_BIT;	 
#define TWI_SCL_IS_HIGH		(TWI_IN_PORT & PC_SCL_BIT )
#define TWI_SCL_IS_LOW		(!(TWI_IN_PORT & PC_SCL_BIT))
*/

// Tiimer Output OCx
#define OC1A_BIT	0x20
#define OC1B_BIT	0x10


#endif //AVRREGISTER
